package com.cedarsoft.spring.rcp;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.context.support.ResourceBundleMessageSource;

import java.text.MessageFormat;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;

/**
 * Message source that logs all requests if they have not been successfully resolved
 */
public class LoggingResourceBundleMessageSource extends ResourceBundleMessageSource {
  //  public class LoggingResourceBundleMessageSource extends ReloadableResourceBundleMessageSource {

  private static final Log log = LogFactory.getLog( LoggingResourceBundleMessageSource.class );

  @NotNull
  private final Set<String> logged = new HashSet<String>();

  @Nullable
  @Override
  protected String resolveCodeWithoutArguments( String code, Locale locale ) {
    String s = super.resolveCodeWithoutArguments( code, locale );
    if ( s == null ) {
      log( code );
    }
    return s;
  }

  @Nullable
  @Override
  protected MessageFormat resolveCode( String code, Locale locale ) {
    MessageFormat messageFormat = super.resolveCode( code, locale );
    if ( messageFormat == null ) {
      log( code );
    }
    return messageFormat;
  }

  private void log( String code ) {
    if ( !logged.contains( code ) ) {
      log.debug( code + "=" );
      logged.add( code );
    }
  }
}
