package com.cedarsoft.spring.rcp;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.apache.commons.lang.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * Represents a property path
 */
public class PropertyPath {
  @NotNull
  @NonNls
  private final List<String> propertyPath = new ArrayList<String>();
  @NotNull
  @NonNls
  private final String property;

  /**
   * Creates a new property path
   *
   * @param propertyPath the property path
   */
  public PropertyPath( @NotNull String... propertyPath ) {
    this( Arrays.asList( propertyPath ) );
  }

  /**
   * Creates a new property path
   *
   * @param propertyPath the property path
   */
  public PropertyPath( @NotNull @NonNls List<? extends String> propertyPath ) {
    this.propertyPath.addAll( propertyPath );
    property = toPropertyName( propertyPath );
  }

  /**
   * Returns the elements of the property path
   *
   * @return the elements
   */
  @Deprecated
  @NotNull
  public List<String> getPropertyPath() {
    return getElements();
  }

  /**
   * Returns the elements
   *
   * @return the elements
   */
  @NotNull
  public List<String> getElements() {
    return Collections.unmodifiableList( propertyPath );
  }

  /**
   * Returns the property path as single string
   *
   * @return the property
   */
  @NotNull
  @NonNls
  public String getProperty() {
    return property;
  }

  @Override
  public String toString() {
    return '{' + property + '}';
  }

  /**
   * Convert the given path to a single string
   *
   * @param propertyPath the property path
   * @return a single string representing the property path
   */
  @NotNull
  public static String toPropertyName( @NotNull @NonNls String... propertyPath ) {
    return toPropertyName( Arrays.asList( propertyPath ) );
  }

  @NotNull
  public static String toPropertyName( @NotNull @NonNls List<? extends String> propertyPath ) {
    return StringUtils.join( propertyPath, '.' );
  }

  @NotNull
  public static PropertyPath create( @NotNull @NonNls String[] propertyPath ) {
    return new PropertyPath( propertyPath );
  }
}
