package com.cedarsoft.spring.rcp.aspects;

import com.cedarsoft.spring.rcp.id.IdStrategy;
import com.cedarsoft.spring.rcp.commands.CommandBuilder;
import com.cedarsoft.spring.rcp.commands.CommandProvider;
import com.cedarsoft.utils.Condition;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.command.ActionCommand;
import org.springframework.richclient.command.config.CommandConfigurer;

/**
 * Abstract aspect that adds/remove behaviour
 *
 * @param <C> the component this aspect is added for
 */
public abstract class AbstractAddRemoveAspect<C> {
  @NotNull
  @NonNls
  public static final String KEY_DELETE_BEAN = ".delete";

  @NotNull
  @NonNls
  public static final String NEW_COMMAND_ID = "newEntry";

  @NotNull
  private final IdStrategy idStrategy;

  @NotNull
  private final CommandBuilder<C> newCommandBuilder = new CommandBuilder<C>();
  @NotNull
  private final CommandBuilder<C> deleteCommandBuilder = new CommandBuilder<C>();

  /**
   * Creates a new aspect
   *
   * @param idStrategy the id strategy
   */
  protected AbstractAddRemoveAspect( @NotNull IdStrategy idStrategy ) {
    this.idStrategy = idStrategy;
  }

  @Nullable
  protected ActionCommand getDeleteCommand( @NotNull C component ) {
    return getDeleteCommandBuilder().getCommand( component );
  }

  /**
   * Returns the new command
   *
   * @param component the component
   * @return the new command
   */
  @Nullable
  protected ActionCommand getNewCommand( @NotNull C component ) {
    return getNewCommandBuilder().getCommand( component );
  }

  @Deprecated
  /**
   * Returns the new command condition
   *
   * @return the new command condition
   */
  @Nullable
  public Condition getNewCommandCondition() {
    return getNewCommandBuilder().getCondition();
  }

  /**
   * Sets the new command condition. If this condition returns false, the new command is *not* added.
   *
   * @param newCommandCondition the condition
   */
  @Deprecated
  public void setNewCommandCondition( @Nullable Condition newCommandCondition ) {
    getNewCommandBuilder().setCondition( newCommandCondition );
  }

  @Deprecated
  @Nullable
  public Condition getDeleteCommandCondition() {
    return getDeleteCommandBuilder().getCondition();
  }

  @Deprecated
  public void setDeleteCommandCondition( @Nullable Condition deleteCommandCondition ) {
    getDeleteCommandBuilder().setCondition( deleteCommandCondition );
  }

  @Deprecated
  @Nullable
  public CommandProvider<C> getNewCommandProvider() {
    return getNewCommandBuilder().getProvider();
  }

  @Deprecated
  public void setNewCommandProvider( @Nullable CommandProvider<C> newCommandProvider ) {
    getNewCommandBuilder().setProvider( newCommandProvider );
  }

  @Deprecated
  @Nullable
  public CommandProvider<C> getDeleteCommandProvider() {
    return getDeleteCommandBuilder().getProvider();
  }

  @Deprecated
  public void setDeleteCommandProvider( @Nullable CommandProvider<C> deleteCommandProvider ) {
    getDeleteCommandBuilder().setProvider( deleteCommandProvider );
  }

  @Deprecated
  @Nullable
  public CommandConfigurer getNewCommandConfigurer() {
    return getNewCommandBuilder().getConfigurer();
  }

  @Deprecated
  public void setNewCommandConfigurer( @Nullable CommandConfigurer newCommandConfigurer ) {
    getNewCommandBuilder().setConfigurer( newCommandConfigurer );
  }

  @Deprecated
  @Nullable
  public CommandConfigurer getDeleteCommandConfigurer() {
    return getDeleteCommandBuilder().getConfigurer();
  }

  @Deprecated
  public void setDeleteCommandConfigurer( @Nullable CommandConfigurer deleteCommandConfigurer ) {
    getDeleteCommandBuilder().setConfigurer( deleteCommandConfigurer );
  }

  /**
   * Prepares and returns the delete command
   *
   * @param component the component
   * @return the delete command (if available)
   */
  @Deprecated
  @Nullable
  protected ActionCommand prepareDeleteCommand( @NotNull C component ) {
    return getDeleteCommand( component );
  }

  /**
   * Prepares and returns the new command
   *
   * @param component the component
   * @return the new command (if available)
   */
  @Deprecated
  @Nullable
  protected ActionCommand prepareNewCommand( @NotNull C component ) {
    return getNewCommand( component );
  }

  @NotNull
  public CommandBuilder<C> getNewCommandBuilder() {
    return newCommandBuilder;
  }

  @NotNull
  public CommandBuilder<C> getDeleteCommandBuilder() {
    return deleteCommandBuilder;
  }

  @NotNull
  public IdStrategy getIdStrategy() {
    return idStrategy;
  }
}
