package com.cedarsoft.spring.rcp.beans;

import org.jetbrains.annotations.NotNull;

/**
 * Offers the ui methods for beans of a type.
 * <p/>
 * BeanUiAccess is often used for beans that are arepresented within a view (based on ObjectTable).
 *
 * @param <T> the type
 */
public interface BeanUiAccess<T> {
  /**
   * Returns the type for this bean ui access
   *
   * @return the type of the bean this offers the ui access for
   */
  @NotNull
  Class<T> getBeanType();

  /**
   * Returns the agent that is able to offer UI based REST access
   *
   * @return the agent
   */
  @NotNull
  BeanAgent<T> getBeanAgent();

  /**
   * Returns the access agent for that type
   *
   * @return the access agent
   */
  @NotNull
  AccessAgent getAccessAgent();

  /**
   * Returns the beans display.
   *
   * @return the beans display
   */
  @NotNull
  BeansDisplay getBeansDisplay();
}
