package com.cedarsoft.spring.rcp.beans;

import com.cedarsoft.spring.rcp.beans.BeanAgent;
import com.cedarsoft.spring.rcp.dialog.FormDialog;
import com.cedarsoft.spring.rcp.dialog.FormDialogFactory;
import com.cedarsoft.CanceledException;
import com.cedarsoft.SwingHelper;
import com.cedarsoft.history.ObservableObjectAccess;
import com.cedarsoft.lookup.Lookup;
import org.jetbrains.annotations.NotNull;

import java.awt.Component;

/**
 * Dialog based agent.
 *
 * @param <T> the type
 */
public class DialogBasedBeanAgent<T> implements BeanAgent<T> {
  @NotNull
  private final FormDialogFactory<T> dialogFactory;

  /**
   * Creates a new dialog based agent
   *
   * @param dialogFactory the dialog factory
   */
  public DialogBasedBeanAgent( @NotNull FormDialogFactory<T> dialogFactory ) {
    this.dialogFactory = dialogFactory;
  }

  @Override
  public void create( @NotNull ObservableObjectAccess<T> objectAccess, @NotNull Lookup context ) throws CanceledException {
    SwingHelper.assertEventThread();

    FormDialog<T> dialog = dialogFactory.createDialog( null, objectAccess );
    showDialog( dialog, context );
  }

  @Override
  public void edit( @NotNull ObservableObjectAccess<T> objectAccess, @NotNull T bean, @NotNull Lookup context ) {
    SwingHelper.assertEventThread();
    showDialog( dialogFactory.createDialog( bean, objectAccess ), context );
  }

  /**
   * Shows the dialog
   *
   * @param dialog  the dialog
   * @param context the context that is used to resolve the parent
   */
  protected void showDialog( @NotNull FormDialog<T> dialog, @NotNull Lookup context ) {
    Component parent = context.lookup( Component.class );
    if ( parent == null ) {
      throw new IllegalArgumentException( "No parent component found in context. A DialogBasedBeanAgent needs a parent component (java.awt.Component) to show a dialog." );
    }

    dialog.setModal( false );
    dialog.setParentComponent( parent );

    dialog.showDialog();
  }
}
