package com.cedarsoft.spring.rcp.beans;

import com.cedarsoft.spring.SpringSupport;
import com.cedarsoft.spring.rcp.beans.BeanAgent;
import com.cedarsoft.spring.rcp.BeanFactory;
import com.cedarsoft.CanceledException;
import com.cedarsoft.SwingHelper;
import com.cedarsoft.history.ObservableObjectAccess;
import com.cedarsoft.lookup.Lookup;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Editor based agent.
 * @param <T> the type
 */
public class EditorBasedBeanAgent<T> implements BeanAgent<T> {
  @Nullable
  private final BeanFactory<T> beanFactory;

  /**
   * Creates a new bean agent
   * @param beanFactory the bean factory
   */
  public EditorBasedBeanAgent( @Nullable BeanFactory<T> beanFactory ) {
    this.beanFactory = beanFactory;
  }

  @Override
  public void create( @NotNull final ObservableObjectAccess<T> objectAccess, @NotNull Lookup context ) throws CanceledException {
    SwingHelper.assertEventThread();

    if ( beanFactory == null ) {
      throw new UnsupportedOperationException( "no bean factory availble" );
    }

    new Thread( new Runnable() {
      @Override
      public void run() {
        assert beanFactory != null;
        T bean = beanFactory.createNewBean();
        objectAccess.add( bean ); //it is necessary to add it early...
        SpringSupport.INSTANCE.openEditor( bean );
      }
    } ).start();
  }

  @Override
  public void edit( @NotNull ObservableObjectAccess<T> objectAccess, @NotNull T bean, @NotNull Lookup context ) {
    SwingHelper.assertEventThread();
    SpringSupport.INSTANCE.openEditor( bean );
  }
}