package com.cedarsoft.spring.rcp.beans.factory;

import com.cedarsoft.spring.rcp.dialog.CreatorFormDialogFactory;
import com.cedarsoft.spring.rcp.creation.CreatorModelFactory;
import com.cedarsoft.spring.rcp.dialog.FormDialogFactory;
import com.cedarsoft.history.ObservableObjectAccess;
import org.jetbrains.annotations.NotNull;

/**
 * UI Factory for business objects that use a creator.
 * This factory is configured using a {@link CreatorDialogBasedUiAccessBuilder.Configuration}.
 * <p/>
 * This factory works based on extended tables, properties dialogs and views.
 * It should be the right thing for most of the default beans
 * @param <T> the type
 */
public class CreatorDialogBasedUiAccessBuilder<T> extends DialogBasedUiAccessBuilder<T> {
  public CreatorDialogBasedUiAccessBuilder( @NotNull Configuration<T> configuration ) {
    super( configuration );
  }

  @Override
  @NotNull
  protected FormDialogFactory<T> createDefaultPropertiesDialogFactory() {
    return new CreatorFormDialogFactory<T>( getIdStrategy(), getConfiguration().getCreatorModelFactory(), configuration.getFormFactory() );
  }

  @NotNull
  @Override
  public Configuration<T> getConfiguration() {
    return ( Configuration<T> ) super.getConfiguration();
  }

  /**
   * Configuration for the UI of business objects.
   * Only this implementation should be necessary for simple business objects
   */
  public interface Configuration<T> extends AbstractConfiguration<T> {
    /**
     * Creates a creator model factory
     *
     * @return the creator model factory
     */
    @NotNull
    CreatorModelFactory<T> getCreatorModelFactory();
  }
}