package com.cedarsoft.spring.rcp.beans.factory;

import com.cedarsoft.spring.rcp.beans.BeanAgent;
import com.cedarsoft.spring.rcp.beans.BeanUiAccess;
import com.cedarsoft.spring.rcp.beans.DialogBasedBeanAgent;
import com.cedarsoft.spring.rcp.dialog.FormDialogFactory;
import org.jetbrains.annotations.NotNull;

/**
 * Creats a {@link BeanUiAccess}.
 *
 * @param <T> the type
 */
public abstract class DialogBasedUiAccessBuilder<T> extends UiAccessBuilder<T> {
  /**
   * Creates a new builder
   *
   * @param configuration the configuration
   */
  protected DialogBasedUiAccessBuilder( @NotNull AbstractConfiguration<T> configuration ) {
    super( configuration );
  }

  private FormDialogFactory<T> formDialogFactory;

  @NotNull
  private FormDialogFactory<T> getPropertiesDialogFactory() {
    if ( formDialogFactory == null ) {
      formDialogFactory = createDefaultPropertiesDialogFactory();
    }
    return formDialogFactory;
  }

  @NotNull
  protected abstract FormDialogFactory<T> createDefaultPropertiesDialogFactory();

  /**
   * Creates the default agent
   *
   * @return the default agent
   */
  @Override
  @NotNull
  protected BeanAgent<T> createAgent() {
    return new DialogBasedBeanAgent<T>( getPropertiesDialogFactory() );
  }
}
