package com.cedarsoft.spring.rcp.binding;

import com.cedarsoft.ObjectAccess;
import com.cedarsoft.history.ElementsChangedEvent;
import com.cedarsoft.history.ElementsCollection;
import com.cedarsoft.history.ElementsListener;
import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanelFactory;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.binding.form.FormModel;
import org.springframework.binding.value.ValueModel;
import org.springframework.richclient.form.binding.support.AbstractBinding;

import javax.swing.JComponent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * the type
 *
 * @param <T>
 */
public class ObjectTablePanelMultiBinding<T> extends AbstractBinding {
  @NotNull
  private ObjectTablePanelFactory<T> objectTablePanelFactory;

  public ObjectTablePanelMultiBinding( @NotNull @NonNls String formPropertyPath, @NotNull FormModel formModel, @NotNull ObjectTablePanelFactory<T> objectTablePanelFactory ) {
    super( formModel, formPropertyPath, null );
    this.objectTablePanelFactory = objectTablePanelFactory;
  }

  @Override
  protected JComponent doBindControl() {
    final ValueModel elementsModel = getListValueModel();
    final ElementsCollection<T> copy = new ElementsCollection<T>( ( Collection<T> ) elementsModel.getValue() );

    final PropertyChangeListener changeListener = new PropertyChangeListener() {
      @Override
      public void propertyChange( PropertyChangeEvent evt ) {
        if ( evt.getPropertyName().equals( "value" ) ) {
          copy.setElements( new ArrayList<T>( ( Collection<T> ) evt.getNewValue() ) );
        }
      }
    };
    elementsModel.addValueChangeListener( changeListener );

    copy.addElementListener( new ElementsListener<T>() {
      @Override
      public void elementsDeleted( @NotNull ElementsChangedEvent<? extends T> event ) {
        elementsModel.setValueSilently( new ArrayList<T>( copy.getElements() ), changeListener );
      }

      @Override
      public void elementsAdded( @NotNull ElementsChangedEvent<? extends T> event ) {
        elementsModel.setValueSilently( new ArrayList<T>( copy.getElements() ), changeListener );
      }

      @Override
      public void elementsChanged( @NotNull ElementsChangedEvent<? extends T> event ) {
        elementsModel.setValueSilently( new ArrayList<T>( copy.getElements() ), changeListener );
      }
    } );

    return objectTablePanelFactory.createPanel( copy );
  }

  /**
   * Returns the value model
   *
   * @return the value model
   */
  @NotNull
  private ValueModel getListValueModel() {
    if ( getValueModel().getValue() instanceof Collection ) {
      return getValueModel();
    }

    if ( getValueModel().getValue() instanceof ObjectAccess ) {
      return getFormModel().getValueModel( formPropertyPath + ".elements", List.class );
    }

    throw new IllegalArgumentException( "Invalid value model type " + getValueModel() );
  }

  @Override
  protected void readOnlyChanged() {
    //todo implement(?)
  }

  @Override
  protected void enabledChanged() {
    //todo implement(?)
  }
}
