package com.cedarsoft.spring.rcp.binding;

import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanel;
import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanelFactory;
import com.cedarsoft.history.ObservableObjectAccess;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.binding.form.FormModel;
import org.springframework.richclient.selection.binding.support.LabelProvider;

import javax.swing.JComponent;

/**
 * Binding to an object table that is shown within a dialog
 *
 * @param <T> the type
 */
public class ObjectTablePanelSingleBinding<T> extends DialogBasedBinding<T> {
  @NotNull
  @NonNls
  private static final String KEY_TITLE = "objectTableDialog.title";
  @NotNull
  @NonNls
  private static final String KEY_DESCRIPTION = "objectTableDialog.description";
  @NotNull
  private final ObservableObjectAccess<T> observableObjectAccess;
  @NotNull
  private final ObjectTablePanelFactory<T> objectTablePanelFactory;

  /**
   * Creates an object table binding
   *
   * @param observableObjectAccess  the observable object access
   * @param labelProvider           the label provider
   * @param objectTablePanelFactory the table based panel factory
   * @param formModel               the form model
   * @param formPropertyPath        the properties path
   * @param nullable                whether it is nullable
   */
  protected ObjectTablePanelSingleBinding( @NotNull final ObservableObjectAccess<T> observableObjectAccess, @NotNull LabelProvider labelProvider, @NotNull final ObjectTablePanelFactory<T> objectTablePanelFactory, @NotNull FormModel formModel, @NotNull @NonNls String formPropertyPath, boolean nullable ) {
    super( formModel, formPropertyPath, null, labelProvider, nullable );
    this.observableObjectAccess = observableObjectAccess;
    this.objectTablePanelFactory = objectTablePanelFactory;
  }

  @NotNull
  private String getPropertyPathLabel() {
    @NonNls
    String[] keys = {formModel.getId() + '.' + formPropertyPath + ".label", formPropertyPath + ".label", formPropertyPath};
    return getMessage( keys );
  }

  @NotNull
  @Override
  protected AbstractDialog createApplicationDialog() {
    ObjectTableDialog dialog = new ObjectTableDialog();

    dialog.setTitle( getMessage( KEY_TITLE, new Object[]{getPropertyPathLabel()} ) );
    dialog.setTitlePaneTitle( getPropertyPathLabel() );
    dialog.setDescription( getMessage( KEY_DESCRIPTION, new Object[]{getPropertyPathLabel()} ) );

    dialog.setModal( false );

    return dialog;
  }

  @NotNull
  public ObservableObjectAccess<T> getObservableObjectAccess() {
    return observableObjectAccess;
  }

  @NotNull
  public ObjectTablePanelFactory<T> getObjectTablePanelFactory() {
    return objectTablePanelFactory;
  }

  /**
   * The dialog that contains the table based panel
   */
  private class ObjectTableDialog extends AbstractDialog {
    @NotNull
    private final ObjectTablePanel<T> panel;

    /**
     * Creates a new dialog
     */
    private ObjectTableDialog() {
      panel = objectTablePanelFactory.createPanel( observableObjectAccess );
    }

    @Override
    protected JComponent createTitledDialogContentPane() {
      return panel;
    }

    @NotNull
    @Override
    protected T getDialogObject() {
      return panel.getObjectTable().getSelectedObject();
    }
  }
}
