package com.cedarsoft.spring.rcp.commands;

import com.cedarsoft.spring.SpringSupport;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationEvent;
import org.springframework.richclient.application.Application;
import org.springframework.richclient.application.ApplicationServices;
import org.springframework.richclient.command.support.ApplicationWindowAwareCommand;
import org.springframework.richclient.factory.ComponentFactory;

/**
 * Abstract base class for window aware commands
 */
public abstract class ExtendedCommand extends ApplicationWindowAwareCommand {
  /**
   * Creates a new command (without an id)
   */
  protected ExtendedCommand() {
  }

  /**
   * Creates a new command
   *
   * @param commandId the command id
   */
  protected ExtendedCommand( @NotNull @NonNls String commandId ) {
    super( commandId );
  }

  @Override
  protected final void doExecuteCommand() {
    try {
      executeCommand();
    } catch ( RuntimeException e ) {
      throw e;
    } catch ( Exception e ) {
      throw new RuntimeException( e );
    }
  }

  protected abstract void executeCommand() throws Exception;

  @NotNull
  protected Application getApplication() {
    return SpringSupport.INSTANCE.getApplication();
  }

  @NotNull
  public <S> S getService( Class<S> serviceType ) {
    return SpringSupport.INSTANCE.getService( serviceType );
  }

  public ApplicationServices getApplicationServices() {
    return SpringSupport.INSTANCE.getApplicationServices();
  }

  @NotNull
  protected String getMessage( @NotNull @NonNls String messageCode, Object... objects ) {
    return SpringSupport.INSTANCE.getMessage( messageCode, objects );
  }

  @NotNull
  protected ApplicationContext getApplicationContext() {
    return SpringSupport.INSTANCE.getApplicationContext();
  }

  public void publishEvent( @NotNull ApplicationEvent event ) {
    SpringSupport.INSTANCE.publishEvent( event );
  }

  public void publishCreated( @NotNull Object object ) {
    SpringSupport.INSTANCE.publishCreated( object );
  }

  public void publishDeleted( @NotNull Object object ) {
    SpringSupport.INSTANCE.publishDeleted( object );
  }

  public void publishModified( @NotNull Object object ) {
    SpringSupport.INSTANCE.publishModified( object );
  }

  public void openEditor( @NotNull Object object ) {
    SpringSupport.INSTANCE.openEditor( object );
  }
}
