package com.cedarsoft.spring.rcp.commands;

import com.cedarsoft.spring.rcp.selection.SelectionModel;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.wizard.Wizard;

/**
 * A wizard command that is based on the currently selected object
 *
 * @param <W> the wizard type
 * @param <T> the type of the selected object
 */
public abstract class SelectionAwareWizardCommand<T, W extends Wizard> extends AbstractWizardCommand<W> {
  @NotNull
  protected final SelectionAwareCommandSupport<T> selectionAwareCommandSupport;

  /**
   * Creates a new wizard command
   *
   * @param commandId      the command id
   * @param selectionModel the selection model
   * @param type           the type
   */
  protected SelectionAwareWizardCommand( @NotNull @NonNls String commandId, @NotNull SelectionModel selectionModel, @NotNull final Class<T> type ) {
    this( commandId, selectionModel, type, null );
  }

  protected SelectionAwareWizardCommand( @NotNull @NonNls String commandId, @NotNull SelectionModel selectionModel, @NotNull final Class<T> type, @Nullable SelectionAwareCommandSupport.Condition<T> condition ) {
    super( commandId );
    selectionAwareCommandSupport = new SelectionAwareCommandSupport<T>( type, selectionModel, this, condition );
  }

  /**
   * Returns the selected object
   *
   * @return the selected object
   */
  @NotNull
  public T getSelectedObject() {
    return selectionAwareCommandSupport.getSelectedObject();
  }

  public void updateEnabledState(  ) {
    selectionAwareCommandSupport.updateEnabledState( this );
  }
}
