package com.cedarsoft.spring.rcp.conversion;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.binding.convert.ConversionContext;
import org.springframework.binding.convert.support.ConversionServiceAwareConverter;

import java.io.File;
import java.io.IOException;

/**
 *
 */
public class FileConverter extends ConversionServiceAwareConverter {
  @Override
  protected Object doConvert( @Nullable Object source, @NotNull Class targetClass, ConversionContext context ) throws Exception {
    if ( targetClass.equals( String.class ) ) {
      return convertToString( ( ( File ) source ) );
    }

    if ( targetClass.equals( File.class ) ) {
      return convertToFile( ( ( String ) source ) );
    }

    throw new IllegalArgumentException( String.valueOf( source ) );
  }

  @Nullable
  private File convertToFile( @Nullable @NonNls String path ) {
    if ( path == null ) {
      return null;
    }

    return new File( path );
  }

  private Object convertToString( @Nullable File file ) throws IOException {
    if ( file == null ) {
      return "";
    }
    return file.getCanonicalPath();
  }

  @Override
  public Class[] getSourceClasses() {
    return new Class[]{String.class, File.class};
  }

  @Override
  public Class[] getTargetClasses() {
    return new Class[]{String.class, File.class};
  }
}
