package com.cedarsoft.spring.rcp.creation;

import com.cedarsoft.spring.rcp.PropertyPath;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.BeansException;
import org.springframework.beans.PropertyAccessor;
import org.springframework.binding.MutablePropertyAccessStrategy;
import org.springframework.binding.support.AbstractPropertyAccessStrategy;
import org.springframework.binding.value.ValueModel;

/**
 * Property access strategy based on a map
 */
public class MappedPropertyAccessStrategy extends AbstractPropertyAccessStrategy {
  @NotNull
  private final MappedPropertyAccessor propertyAccessor;

  public MappedPropertyAccessStrategy( @NotNull Class<?> type ) {
    super( new Object() );
    propertyAccessor = new MappedPropertyAccessor( type );
  }

  /**
   * Registers a property
   *
   * @param propertyPath the names of the property
   * @param propertyType the type of the property
   */
  public void registerProperty( @NotNull Class<?> propertyType, @NotNull @NonNls PropertyPath propertyPath ) {
    propertyAccessor.registerProperty( propertyType, propertyPath );
  }

  public void registerProperty( @NotNull Class<?> propertyType, @NotNull @NonNls String property ) {
    propertyAccessor.registerProperty( propertyType, property );
  }

  /**
   * Registers a property and sets a default value
   *
   * @param propertyName the property name
   * @param defaultValue the default value that is set
   */
  public <T> void registerProperty( @NotNull T defaultValue, @NotNull @NonNls String propertyName ) {
    registerProperty( defaultValue.getClass(), propertyName );
    getPropertyAccessor().setPropertyValue( propertyName, defaultValue );
  }

  public <T> void registerProperty( @NotNull T defaultValue, @NotNull PropertyPath propertyPath ) {
    registerProperty( defaultValue, propertyPath.getProperty() );
  }

  public <T> void registerProperty( @NotNull Class<? super T> type, @NotNull T defaultValue, @NotNull @NonNls String propertyName ) {
    registerProperty( type, propertyName );
    getPropertyAccessor().setPropertyValue( propertyName, defaultValue );
  }

  @NotNull
  @Override
  protected PropertyAccessor getPropertyAccessor() {
    return propertyAccessor;
  }

  @Override
  @NotNull
  public MutablePropertyAccessStrategy getPropertyAccessStrategyForPath( @NotNull @NonNls String propertyPath ) throws BeansException {
    throw new UnsupportedOperationException();
  }

  @Override
  public MutablePropertyAccessStrategy newPropertyAccessStrategy( @NotNull ValueModel domainObjectHolder ) {
    throw new UnsupportedOperationException();
  }

  @Override
  protected void domainObjectChanged() {
  }
}
