package com.cedarsoft.spring.rcp.dialog;

import com.cedarsoft.CommitableObjectAccess;
import com.cedarsoft.spring.rcp.BeanFactory;
import com.cedarsoft.spring.rcp.forms.DefaultFormBackend;
import com.cedarsoft.spring.rcp.forms.FormFactory;
import com.cedarsoft.spring.rcp.id.IdStrategy;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Dialog factory that creates a dialog for a special business object
 *
 * @param <T> the type
 */
public class BeanFormDialogFactory<T> implements FormDialogFactory<T> {
  @NotNull
  @NonNls
  private final String dialogId;
  @NotNull
  @NonNls
  private final String formId;

  @Nullable
  private final BeanFactory<T> beanFactory;
  @NotNull
  private final FormFactory<T> formFactory;

  /**
   * Creates a new factory
   *
   * @param idStrategy  the id strategy that is used to created the dialog and form id
   * @param beanFactory the bean factory
   * @param formFactory the form factory
   */
  public BeanFormDialogFactory( @NotNull IdStrategy idStrategy, @Nullable  BeanFactory<T> beanFactory, @NotNull FormFactory<T> formFactory ) {
    this( idStrategy.createDialogId(), idStrategy.createFormId(), beanFactory, formFactory );
  }

  /**
   * Creates a  new factory
   *
   * @param dialogId    the id
   * @param formId      the form id
   * @param beanFactory the bean factory that is used to create new beans (if necessary)
   * @param formFactory the form factory that creates forms for the given bean type
   */
  public BeanFormDialogFactory( @NotNull String dialogId, @NotNull String formId, @Nullable  BeanFactory<T> beanFactory, @NotNull FormFactory<T> formFactory ) {
    this.dialogId = dialogId;
    this.formId = formId;
    this.beanFactory = beanFactory;
    this.formFactory = formFactory;
  }

  @Override
  @NotNull
  public FormDialog<T> createDialog( @Nullable T selectedObject, @NotNull CommitableObjectAccess<T> objectAccess ) {
    return new FormDialog<T>( dialogId, new DefaultFormBackend<T>( formId, objectAccess, beanFactory, selectedObject ), formFactory );
  }
}
