package com.cedarsoft.spring.rcp.forms;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.binding.form.FormModel;
import org.springframework.richclient.form.FormModelHelper;

/**
 * Simple backend with a static form model.
 *
 * @param <T> the bean type
 */
public abstract class AbstractStaticFormBackend<T> implements FormBackend {
  @NotNull
  private final FormModel formModel;

  private boolean shallAskForCommitConfirmation;

  /**
   * Creates a new model for the given object
   *
   * @param formId the form id
   * @param object the object
   */
  protected AbstractStaticFormBackend( @NotNull @NonNls String formId, @NotNull T object ) {
    this( FormModelHelper.createFormModel( object, formId ) );
  }

  /**
   * Creates a new backend
   *
   * @param formModel the form model
   */
  protected AbstractStaticFormBackend( @NotNull FormModel formModel ) {
    this.formModel = formModel;
    assert formModel.getId() != null;
  }

  /**
   * Returns the form object
   *
   * @return the form object
   */
  @NotNull
  public T getFormObject() {
    return ( T ) getFormModel().getFormObject();
  }

  @Override
  public boolean isCreatingNew() {
    return false;
  }

  @Override
  @NotNull
  public FormModel getFormModel() {
    return formModel;
  }

  @Override
  public boolean shallAskForCommitConfirmation() {
    return shallAskForCommitConfirmation;
  }

  public void setShallAskForCommitConfirmation( boolean shallAskForCommitConfirmation ) {
    this.shallAskForCommitConfirmation = shallAskForCommitConfirmation;
  }
}
