package com.cedarsoft.spring.rcp.forms;

import com.cedarsoft.CanceledException;
import com.cedarsoft.CommitableObjectAccess;
import com.cedarsoft.spring.rcp.BeanFactory;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Default implementation of form backend.
 * This implementation should not be subclasses - it uses delegation...
 *
 * @param <T> the type
 */
public class DefaultFormBackend<T> extends ObjectAccessBasedFormBackend<T> {
  @Nullable
  private final BeanFactory<T> beanFactory;

  /**
   * Creates a new default form backend
   *
   * @param formId       the form id
   * @param objectAccess the object access
   * @param beanFactory  the bean factory that is used to create a new object (if creating new)
   * @param object       the object (if there is any, if creating a new object the given value is null)
   */
  public DefaultFormBackend( @NotNull @NonNls String formId, @NotNull CommitableObjectAccess<T> objectAccess, @Nullable BeanFactory<T> beanFactory, @Nullable T object ) {
    super( formId, objectAccess, object );
    this.beanFactory = beanFactory;
  }

  @Override
  @NotNull
  protected T createNewObject() throws CanceledException, UnsupportedOperationException {
    if ( beanFactory == null ) {
      throw new UnsupportedOperationException();
    }
    return beanFactory.createNewBean();
  }
}
