package com.cedarsoft.spring.rcp.forms;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.springframework.binding.form.ValidatingFormModel;
import org.springframework.richclient.form.FormModelHelper;

/**
 * Default form model factory
 *
 * @param <T> the type
 */
public class DefaultFormModelFactory<T> implements FormModelFactory<T> {
  private boolean createUnbufferedModel;

  @NotNull
  @NonNls
  private final String id;

  /**
   * Creates a new default form model factory
   *
   * @param id
   */
  public DefaultFormModelFactory( @NotNull @NonNls String id ) {
    this( id, false );
  }

  /**
   * Creates a new default form model factory
   *
   * @param id
   * @param createUnbufferedModel whether an unbuffered model is created
   */
  public DefaultFormModelFactory( @NotNull @NonNls String id, boolean createUnbufferedModel ) {
    this.createUnbufferedModel = createUnbufferedModel;
    this.id = id;
  }

  @Override
  @NotNull
  public ValidatingFormModel createFormModel( @NotNull T formObject ) {
    if ( createUnbufferedModel ) {
      return FormModelHelper.createUnbufferedFormModel( formObject, id );
    } else {
      return FormModelHelper.createFormModel( formObject, id );
    }
  }

  /**
   * Whether this factory creates unbuffered models
   *
   * @return whether this factory creates unbuffered models
   */
  public boolean isCreateUnbufferedModel() {
    return createUnbufferedModel;
  }

  public void setCreateUnbufferedModel( boolean createUnbufferedModel ) {
    this.createUnbufferedModel = createUnbufferedModel;
  }
}
