package com.cedarsoft.spring.rcp.forms;

import org.jetbrains.annotations.NotNull;
import org.springframework.binding.value.ValueModel;

import javax.swing.JComponent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * Connects the enabled state of a component with the given value model (that returns Booleans)
 */
public class EnableBridge {
  private EnableBridge() {
  }

  /**
   * Connects the enabled state of the component with the given value model (that must return Booleans)
   *
   * @param valueModel the value model (that returns a boolean)
   * @param component  the component
   */
  public static void connect( @NotNull ValueModel valueModel, @NotNull final JComponent component ) {
    component.setEnabled( ( Boolean ) valueModel.getValue() );
    valueModel.addValueChangeListener( new PropertyChangeListener() {
      @Override
      public void propertyChange( PropertyChangeEvent evt ) {
        component.setEnabled( ( Boolean ) evt.getNewValue() );
      }
    } );
  }
}
