package com.cedarsoft.spring.rcp.forms;

import org.fest.reflect.core.Reflection;
import org.jetbrains.annotations.NotNull;
import org.springframework.binding.form.FormModel;

/**
 * Creates an extended form using reflection.
 * There must exist a constructor with a FormModel as only argument.
 *
 * @param <T>
 */
public class ReflectionFormFactory<T> implements FormFactory<T> {
  @NotNull
  public static <T> ReflectionFormFactory<T> create( @NotNull Class<? extends ExtendedForm<T>> formType ) {
    return new ReflectionFormFactory<T>( formType );
  }

  @NotNull
  private final Class<? extends ExtendedForm<T>> formType;

  /**
   * Creates a new reflection form factory
   *
   * @param formType the form type
   */
  public ReflectionFormFactory( @NotNull Class<? extends ExtendedForm<T>> formType ) {
    this.formType = formType;
    //Ensure the constructor exists
    Reflection.constructor().withParameterTypes( FormModel.class ).in( formType );
  }

  @Override
  @NotNull
  public ExtendedForm<T> createForm( @NotNull FormModel formModel ) {
    return Reflection.constructor().withParameterTypes( FormModel.class ).in( formType ).newInstance( formModel );
  }
}
