package com.cedarsoft.spring.rcp.forms;

import org.jetbrains.annotations.NotNull;
import org.springframework.binding.form.FormModel;

/**
 * Static form factory that returns always the same form instance
 *
 * @param <T> the type
 */
public class StaticFormFactory<T> implements FormFactory<T> {
  @NotNull
  private final ExtendedForm<? extends T> form;

  /**
   * Creates a new static form factory
   *
   * @param form the form
   */
  public StaticFormFactory( @NotNull ExtendedForm<? extends T> form ) {
    this.form = form;
    assert form.getId() != null;
  }

  @Override
  @NotNull
  public ExtendedForm<? extends T> createForm( @NotNull FormModel formModel ) {
    if ( form.getFormModel() != formModel ) {
      throw new IllegalStateException( "Invalid form model. Was " + form.getFormModel() + " but should be " + formModel );
    }
    return form;
  }
}
