package com.cedarsoft.spring.rcp.hierarchy;

import com.cedarsoft.lookup.Lookup;
import com.cedarsoft.utils.Cache;
import com.cedarsoft.utils.HashedCache;
import org.jetbrains.annotations.NotNull;

import java.util.WeakHashMap;

/**
 * Creates details for each parent and caches the result.
 */
public abstract class CachingTitleProvider<P> implements TitleProvider<P> {
  @NotNull
  private final Cache<P, Lookup> cache = new HashedCache<P, Lookup>( new WeakHashMap<P, Lookup>(), new Cache.Factory<P, Lookup>() {
    @Override
    @NotNull
    public Lookup create( @NotNull P key ) {
      return createTitle( key );
    }
  } );

  @Override
  @NotNull
  public Lookup getTitle( @NotNull P parent ) {
    return cache.get( parent );
  }

  public void invalidateCache( @NotNull P parent ) {
    cache.remove( parent );
  }

  /**
   * Creates the details for the given parent
   *
   * @param parent the parent
   * @return the details
   */
  @NotNull
  protected abstract Lookup createTitle( @NotNull P parent );
}