package com.cedarsoft.spring.rcp.prefetch;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.springframework.util.StopWatch;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.Set;

import com.cedarsoft.spring.rcp.prefetch.ObjectPrefetcher;

/**
 * Prefetches objects
 */
public class Prefetcher {
  private static final Log log = LogFactory.getLog( Prefetcher.class );

  @NotNull
  private final List<ObjectPrefetcher<Object>> prefetchers = new ArrayList<ObjectPrefetcher<Object>>();
  @NotNull
  private final ObjectPrefetcher<Object> fallbackObjectPrefetcher;

  /**
   * Creates a new prefetcher
   *
   * @param prefetchers              the prefetchers that are used
   * @param fallbackObjectPrefetcher the fallback prefetcher
   */
  public Prefetcher( @NotNull List<? extends ObjectPrefetcher<Object>> prefetchers, @NotNull ObjectPrefetcher<Object> fallbackObjectPrefetcher ) {
    this.fallbackObjectPrefetcher = fallbackObjectPrefetcher;
    this.prefetchers.addAll( prefetchers );
  }

  /**
   * Prefetches the given object
   *
   * @param object the object that is prefetched
   */
  public void prefetch( @NotNull Object object ) {
    StopWatch stopWatch = new StopWatch( "Prefetching" );

    Queue<Object> queue = new LinkedList<Object>();
    queue.add( object );
    Set<Object> prefetched = new HashSet<Object>();

    while ( !queue.isEmpty() ) {
      Object current = queue.poll();
      if ( current == null ) {
        continue;
      }
      prefetched.add( current );

      stopWatch.start( "Calling Prefetcher for " + current );
      for ( ObjectPrefetcher<Object> prefetcher : findObjectPrefetcher( current ) ) {
        queue.addAll( prefetcher.prefetch( current ) );
        queue.removeAll( prefetched );
      }

      stopWatch.stop();
    }

    log.debug( stopWatch.prettyPrint() );
  }

  @NotNull
  private Collection<ObjectPrefetcher<Object>> findObjectPrefetcher( @NotNull Object current ) {
    List<ObjectPrefetcher<Object>> responsibles = new ArrayList<ObjectPrefetcher<Object>>();

    for ( ObjectPrefetcher<Object> prefetcher : prefetchers ) {
      if ( prefetcher.isResponsible( current ) ) {
        responsibles.add( prefetcher );
      }
    }

    if ( responsibles.isEmpty() ) {
      responsibles.add( fallbackObjectPrefetcher );
    }

    return responsibles;
  }

}