package com.cedarsoft.spring.rcp.selection;

import com.cedarsoft.history.DelegatingObjectAccess;
import com.cedarsoft.history.ObservableObjectAccess;
import com.cedarsoft.lookup.Lookup;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * An object access that is selection aware
 *
 * @param <T> the type
 */
public class SelectionAwareObjectAccess<T> extends DelegatingObjectAccess<T> {
  @NotNull
  private final SelectionModel selectionModel;

  /**
   * Creates a new object access
   *
   * @param selectionModel   the selection model
   * @param delegateResolver the delegating resolver
   */
  public SelectionAwareObjectAccess( @NotNull SelectionModel selectionModel, @NotNull final DelegateResolver<T> delegateResolver ) {
    super( delegateResolver.resolve( selectionModel.getCurrentlySelectedObject() ) );
    this.selectionModel = selectionModel;

    selectionModel.addSelectionChangeListener( new SelectionChangeListener() {
      @Override
      public void selectionChanged( @NotNull Lookup selectedObjects ) {
        setCurrentDelegate( delegateResolver.resolve( selectedObjects ) );
      }
    } );
  }

  @NotNull
  public SelectionModel getSelectionModel() {
    return selectionModel;
  }

  /**
   * Resolves the delegate
   *
   * @param <T> the type
   */
  public interface DelegateResolver<T> {
    /**
     * Resolve the delegate from the currently selected object
     *
     * @param currentlySelectedObject the currently selected objects
     * @return the delegate (if there is one selected)
     */
    @Nullable
    ObservableObjectAccess<T> resolve( @NotNull Lookup currentlySelectedObject );
  }
}
