package com.cedarsoft.spring.rcp.table;

import com.cedarsoft.spring.SpringSupport;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.application.statusbar.StatusBar;

/**
 * Status bar support for the extended object table
 */
public class StatusBarSupport {
  @NotNull
  @NonNls
  public static final String SHOWINGALL_MSG_KEY = "objectTable.showingAll.message";
  @NotNull
  @NonNls
  public static final String SHOWINGN_MSG_KEY = "objectTable.showingN.message";
  @NotNull
  @NonNls
  public static final String SELECTEDN_MSG_KEY = "objectTable.selectedN.message";


  @NotNull
  private final String objectSingularName;

  @NotNull
  private final String objectPluralName;

  @NotNull
  private final String modelId;

  @Nullable
  private StatusBar currentStatusBar;

  public StatusBarSupport( @NotNull @NonNls String modelId ) {
    this.modelId = modelId;

    objectSingularName = SpringSupport.INSTANCE.getMessage( modelId + ".objectName.singular" );
    objectPluralName = SpringSupport.INSTANCE.getMessage( modelId + ".objectName.plural" );
  }

  @Nullable
  public StatusBar getCurrentStatusBar() {
    return currentStatusBar;
  }

  public void setCurrentStatusBar( @Nullable StatusBar currentStatusBar ) {
    this.currentStatusBar = currentStatusBar;
  }

  /**
   * Update the status bar with the current display counts.
   *
   * @param all         the amount of all available rows
   * @param showing     the amount of shown rows
   * @param selectedRow the currently selected row (or -1 if no row is selected)
   */
  protected void updateStatusBar( int all, int showing, int selectedRow ) {
    if ( currentStatusBar != null ) {
      String msg;
      if ( all == showing ) {
        Object[] args = {String.valueOf( all ), all == 1 ? objectSingularName : objectPluralName};
        msg = SpringSupport.INSTANCE.getMessage( modelId + '.' + SHOWINGALL_MSG_KEY, args );
      } else {
        Object[] args = {String.valueOf( showing ), showing == 1 ? objectSingularName : objectPluralName, String.valueOf( all )};
        msg = SpringSupport.INSTANCE.getMessage( modelId + '.' + SHOWINGN_MSG_KEY, args );
      }

      // Now add the selection info
      if ( selectedRow > 0 ) {
        msg += SpringSupport.INSTANCE.getMessage( modelId + '.' + SELECTEDN_MSG_KEY, String.valueOf( selectedRow ) );
      }

      if ( currentStatusBar != null ) {
        currentStatusBar.setMessage( msg );
      }
    }
  }
}
