package com.cedarsoft.spring.rcp.tbpanel.aspects;

import com.cedarsoft.spring.rcp.commands.CommandProvider;
import com.cedarsoft.spring.rcp.editor.OpenInEditorCommand;
import com.cedarsoft.spring.rcp.table.ExtendedObjectTable;
import com.cedarsoft.spring.rcp.table.ObjectTable;
import com.cedarsoft.spring.rcp.tbpanel.ObjectTablePanel;
import com.cedarsoft.spring.rcp.tbpanel.config.DoubleClickConfigurer;
import org.jetbrains.annotations.NotNull;
import org.springframework.richclient.command.ActionCommand;

/**
 * This aspect adds an double click handler that opens the selected object within the editor
 *
 * @param <T> the type
 */
public class DoubleClickOpensEditorAspect<T> implements ObjectTablePanelAspect<T> {
  @Override
  public void apply( @NotNull ObjectTablePanel<T> panel ) {
    panel.addConfigurer( new DoubleClickConfigurer<T>( new CommandProvider<ObjectTable<T>>() {
      @Override
      @NotNull
      public ActionCommand getCommand( @NotNull final ObjectTable<T> object ) {
        return new OpenInEditorCommand() {
          @Override
          @NotNull
          protected Object getCurrentObject() {
            return object.getSelectedObject();
          }
        };
      }
    } ) );
  }
}
