package com.cedarsoft.spring.rcp.tbpanel.config;

import com.cedarsoft.spring.rcp.commands.CommandProvider;
import com.cedarsoft.spring.rcp.table.ExtendedObjectTable;
import com.cedarsoft.spring.rcp.table.ObjectTable;
import org.jetbrains.annotations.NotNull;
import org.springframework.richclient.command.ActionCommand;

/**
 * Configures the double click command.
 *
 * @param <T> the type
 */
public class DoubleClickConfigurer<T> implements ObjectTableConfigurer<T> {
  @NotNull
  private final CommandProvider<ObjectTable<T>> doubleClickCommandProvider;

  /**
   * Creates a new double click configurer
   *
   * @param command the command that is executed on double click
   */
  public DoubleClickConfigurer( @NotNull final ActionCommand command ) {
    this( new CommandProvider<ObjectTable<T>>() {
      @Override
      @NotNull
      public ActionCommand getCommand( @NotNull ObjectTable<T> object ) {
        return command;
      }
    } );
  }

  /**
   * Creates a new configurer
   *
   * @param doubleClickCommandProvider the provider that provides the double click command
   */
  public DoubleClickConfigurer( @NotNull CommandProvider<ObjectTable<T>> doubleClickCommandProvider ) {
    this.doubleClickCommandProvider = doubleClickCommandProvider;
  }

  @Override
  public void configurePreShow( @NotNull ObjectTable<T> objectTable ) {
  }

  @Override
  public void configurePostShow( @NotNull ObjectTable<T> objectTable ) {
    configureDoubleClickHandler( objectTable );
  }

  /**
   * Configures the double click handler
   *
   * @param objectTable the object table
   */
  protected void configureDoubleClickHandler( @NotNull ObjectTable<T> objectTable ) {
    objectTable.setDoubleClickHandler( doubleClickCommandProvider.getCommand( objectTable ) );
  }
}
