package com.cedarsoft.spring.rcp.tree.layered;

import com.cedarsoft.spring.rcp.tree.ChildProvider;
import com.cedarsoft.spring.rcp.tree.layered.config.PopupConfigurer;
import com.cedarsoft.history.ObservableCollection;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.richclient.command.CommandGroup;

import javax.swing.JPopupMenu;
import javax.swing.JTree;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Default layer configuration.
 */
public class DefaultTreeLayer<P, C> implements TreeLayer<P, C> {
  @Nullable
  private ChildProvider<P, C> childProvider;

  @Override
  @Nullable
  public ObservableCollection<C> getChildren( @NotNull P parent ) {
    if ( childProvider == null ) {
      return null;
    }
    return childProvider.getChildren( parent );
  }

  @Override
  public boolean isLeaf( @NotNull P parent ) {
    return childProvider == null;
  }

  @Override
  public void handleDoubleClick( @NotNull P parent, @NotNull C clickedChild, @NotNull MouseEvent event ) {
  }

  @NotNull
  private final List<PopupConfigurer<P, C>> popupConfigurers = new ArrayList<PopupConfigurer<P, C>>();

  @Override
  @Nullable
  public JPopupMenu createPopupMenu( @NotNull P parent, @NotNull C selectedChild, @NotNull MouseEvent e ) {
    if ( popupConfigurers.isEmpty() ) {
      return null;
    }

    CommandGroup group = new CommandGroup();

    for ( Iterator<PopupConfigurer<P, C>> it = popupConfigurers.iterator(); it.hasNext(); ) {
      PopupConfigurer<P, C> popupConfigurer = it.next();
      popupConfigurer.configure( group, parent, selectedChild, ( JTree ) e.getSource() ); //todo use MouseEvent(?)
      if ( it.hasNext() ) {
        group.addSeparator();
      }
    }

    return group.createPopupMenu();
  }

  public void addPopupConfigurer( @NotNull PopupConfigurer<P, C> configurer ) {
    this.popupConfigurers.add( configurer );
  }

  @Nullable
  public ChildProvider<P, C> getChildProvider() {
    return childProvider;
  }

  public void setChildProvider( @Nullable ChildProvider<P, C> childProvider ) {
    this.childProvider = childProvider;
  }
}
