package com.cedarsoft.spring.rcp.tree.layered;

import com.cedarsoft.history.ObservableCollection;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import javax.swing.JPopupMenu;
import java.awt.event.MouseEvent;

/**
 * @param <P> The type of the parent
 * @param <C> The type of the children
 */
public interface TreeLayer<P, C> {
  /**
   * Returns the children for the parent
   *
   * @param parent the parent
   * @return the children
   */
  @Nullable
  ObservableCollection<C> getChildren( @NotNull P parent );

  /**
   * Returns whether this is a leaf
   *
   * @param parent the parent
   * @return whether it is a leaf
   */
  boolean isLeaf( @NotNull P parent );

  /**
   * Handles the double click
   *
   * @param parent       the parent that is double clicked...
   * @param clickedChild the clicked child
   * @param event the event
   */
  void handleDoubleClick( @NotNull P parent, @NotNull C clickedChild, @NotNull MouseEvent event );

  /**
   * Creates the popup menu
   *
   * @param parent        the parent
   * @param selectedChild the selected child
   * @param e             the mouse event
   * @return the popup menu or null if no popup menu shall be shown
   */
  @Nullable
  JPopupMenu createPopupMenu( @NotNull P parent, @NotNull C selectedChild, @NotNull MouseEvent e );
}