package com.cedarsoft.spring.rcp.view;

import com.cedarsoft.spring.rcp.views.ExtendedView;
import org.jetbrains.annotations.NotNull;
import org.springframework.richclient.application.docking.jide.view.JideViewDescriptor;

import java.util.Map;

/**
 * Special view factory that is based on a ViewConfiguration
 */
public class JideViewDescriptorFactory extends AbstractViewDescriptorFactory<JideViewDescriptor> {
  public JideViewDescriptorFactory( @NotNull Class<? extends ExtendedView> viewClass, @NotNull Map<String, Object> viewProperties ) {
    super( viewClass, viewProperties );
  }

  /**
   * Creates a new factory
   *
   * @param viewConfiguration the view configuration
   */
  public JideViewDescriptorFactory( @NotNull ViewConfiguration viewConfiguration ) {
    super( viewConfiguration );
  }

  @NotNull
  @Override
  protected JideViewDescriptor createPlainDescriptor() {
    return new JideViewDescriptor();
  }

  @Override
  protected void addCustomInitialization( @NotNull JideViewDescriptor descriptor ) {

    descriptor.setInitMode( initMode );
    descriptor.setAutohidable( autohidable );
    descriptor.setDockable( dockable );
    descriptor.setFloatable( floatable );
    if ( floatBounds != null ) {
      descriptor.setFloatBounds( floatBounds );
    }
    descriptor.setFloatOnShow( floatOnShow );
    descriptor.setHidable( hidable );
    descriptor.setInitIndex( initIndex );
    descriptor.setInitMode( initMode );
    descriptor.setInitSide( initSide );
    descriptor.setMaximizable( maximizable );
    descriptor.setRearrangable( rearrangable );
    descriptor.setShowGripper( showGripper );
    descriptor.setShowTitleBar( showTitleBar );
    descriptor.setSideDockAllowed( sideDockAllowed );
    descriptor.setSlidingAutohide( slidingAutohide );
    descriptor.setTabDockAllowed( tabDockAllowed );

    if ( alwaysVisible ) {
      descriptor.setRearrangable( false );
      descriptor.setHidable( false );
      descriptor.setMaximizable( false );
      descriptor.setAutohidable( false );
      descriptor.setFloatable( false );
    }
  }
}
