package com.cedarsoft.utils.springrcp.beanlist;

import org.jetbrains.annotations.NotNull;

/**
 * A BeanManager is a special helper class needed to display
 * the values within a beanlist structure.
 * <p/>
 * <p/>
 * <p/>
 * Date: 18.08.2006<br>
 * Time: 16:03:34<br>
 *
 * @author <a href="http://johannes-schneider.info">Johannes Schneider</a> -
 *         <a href="http://www.xore.de">Xore Systems</a>
 */
public interface BeanManager<T> {
  /**
   * Returns the index (row in the beanlist) of the given bean
   *
   * @param bean the bean the index is searched for
   * @return the index
   */
  int indexOf( @NotNull T bean );

  /**
   * Return the index for the given row
   *
   * @param index the index
   * @return the bean
   */
  @NotNull
  T getBean( int index );

  /**
   * Create a new bean. This bean is added later ({@link #add(Object)}) .
   *
   * @return the newly created bean
   */
  @NotNull
  T createNewBean();

  /**
   * Add the bean.
   * This method is called after {@link #createNewBean()}.
   *
   * @param bean the bean that shall be added.
   */
  void add( @NotNull T bean );

  /**
   * Delete the given bean
   *
   * @param bean the bean that shall be deleted
   */
  void delete( @NotNull T bean );

  /**
   * Save the given bean
   *
   * @param bean the bean that shall be saved
   */
  void save( @NotNull T bean );
}