package com.cedarsoft.utils.springrcp.beanlist.command;

import com.cedarsoft.utils.springrcp.beanlist.BeanListCommandFactory;
import com.cedarsoft.utils.springrcp.beanlist.BeanListView;
import org.jetbrains.annotations.NotNull;
import org.springframework.binding.value.ValueModel;
import org.springframework.richclient.command.AbstractCommand;
import org.springframework.richclient.list.ListSelectionValueModelAdapter;
import org.springframework.richclient.list.ListSingleSelectionGuard;

/**
 * <p/>
 * Date: 29.08.2006<br>
 * Time: 14:57:34<br>
 *
 * @author <a href="http://johannes-schneider.info">Johannes Schneider</a> -
 *         <a href="http://www.xore.de">Xore Systems</a>
 */
public class DefaultBeanListCommandFactory<T> implements BeanListCommandFactory {
  private AbstractCommand deleteCommand;
  private AbstractCommand propertiesCommand;
  private AbstractCommand newCommand;
  private final BeanListView<T> listView;

  private boolean initialized;

  public DefaultBeanListCommandFactory( @NotNull BeanListView<T> listView ) {
    this.listView = listView;
  }

  protected void initializeCommands() {
    initialized = true;
    deleteCommand = new DeleteCommand<T>( listView );
    propertiesCommand = new PropertiesCommand<T>( listView );
    newCommand = new NewCommand<T>( listView );
  }

  @Override
  @NotNull
  public AbstractCommand getDeleteCommand() {
    if ( !initialized ) {
      initializeCommands();
    }
    return deleteCommand;
  }

  @Override
  @NotNull
  public AbstractCommand getPropertiesCommand() {
    if ( !initialized ) {
      initializeCommands();
    }
    return propertiesCommand;
  }

  @Override
  @NotNull
  public AbstractCommand getNewCommand() {
    if ( !initialized ) {
      initializeCommands();
    }
    return newCommand;
  }

  @Override
  public void initialize() {
    ValueModel selectionHolder = new ListSelectionValueModelAdapter( listView.getTable().getSelectionModel() );

    //noinspection ResultOfObjectAllocationIgnored
    new ListSingleSelectionGuard( selectionHolder, deleteCommand );
    //noinspection ResultOfObjectAllocationIgnored
    new ListSingleSelectionGuard( selectionHolder, propertiesCommand );

    newCommand.setEnabled( true );
  }
}
