package com.cedarsoft.utils.springrcp.selection;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

/**
 * <p/>
 * Date: 22.08.2006<br>
 * Time: 20:35:19<br>
 *
 * @author <a href="http://johannes-schneider.info">Johannes Schneider</a> -
 *         <a href="http://www.xore.de">Xore Systems</a>
 */
public class DefaultSelection<T> implements Selection<T> {
  private Class<T> type;
  private Collection<T> selectedElements;
  private T selected;

  public DefaultSelection( @NotNull Class<T> type, @NotNull Collection<T> selectedElements ) {
    this.type = type;
    this.selectedElements = new ArrayList<T>( selectedElements );
  }

  public DefaultSelection( @NotNull Class<T> type ) {
    this( type, ( T ) null );
  }

  public DefaultSelection( @NotNull Class<T> type, @Nullable T selected ) {
    this.type = type;
    this.selected = selected;
    this.selectedElements = Collections.emptyList();
  }

  public boolean isEmty() {
    return selectedElements.isEmpty() && selected == null;
  }

  public boolean isSingleSelection() {
    return selected != null;
  }

  public boolean isMultiSelection() {
    return !selectedElements.isEmpty();
  }

  @NotNull
  public T getSingleSelection() {
    return selected;
  }

  @NotNull
  public Collection<T> getMultiSelection() {
    return Collections.unmodifiableCollection( selectedElements );
  }

  @NotNull
  public Class<T> getType() {
    return type;
  }

  @Override
  public String toString() {
    return "DefaultSelection{" +
        "type=" + type +
        ", selectedElements=" + selectedElements +
        ", selected=" + selected +
        '}';
  }
}
